@@ -57,8 +57,7 @@
 #define USB_PID_DNP_DS40  0x0003 // Also Citizen CX
 #define USB_PID_DNP_DS80  0x0004 // Also Citizen CX-W, and Mitsubishi CP-3800DW
 #define USB_PID_DNP_DSRX1 0x0005 // Also Citizen CY
-#define USB_PID_DNP_DS80D 0x0007
-#define USB_PID_DNP_DS620_OLD 0x0008
+#define USB_PID_DNP_DS80D 0x0008
 
 #define USB_PID_CITIZEN_CW02 0x0006 // Also OP900II
 #define USB_PID_CITIZEN_CX02 0x000A
@@ -612,7 +611,7 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
 		int len = 0;
 
 		/* Query Duplex Media Info */
-		dnpds40_build_cmd(&cmd, "INFO", "CUT_PAPER", 0);
+		dnpds40_build_cmd(&cmd, "INFO", "UNIT_CUT_PAPER", 0);
 
 		resp = dnpds40_resp_cmd(ctx, &cmd, &len);
 		if (resp) {
@@ -2550,7 +2549,7 @@ static int dnpds40_cmdline_arg(void *vctx, int argc, char **argv)
 /* Exported */
 struct dyesub_backend dnpds40_backend = {
 	.name = "DNP DS40/DS80/DSRX1/DS620/DS820",
-	.version = "0.97",
+	.version = "0.97.2",
 	.uri_prefix = "dnpds40",
 	.cmdline_usage = dnpds40_cmdline,
 	.cmdline_arg = dnpds40_cmdline_arg,
@@ -2564,7 +2563,6 @@ struct dyesub_backend dnpds40_backend = {
 	{ USB_VID_CITIZEN, USB_PID_DNP_DS40, P_DNP_DS40, NULL},
 	{ USB_VID_CITIZEN, USB_PID_DNP_DS80, P_DNP_DS80, NULL},
 	{ USB_VID_CITIZEN, USB_PID_DNP_DSRX1, P_DNP_DSRX1, NULL},
-	{ USB_VID_CITIZEN, USB_PID_DNP_DS620_OLD, P_DNP_DS620, NULL},
 	{ USB_VID_DNP, USB_PID_DNP_DS620, P_DNP_DS620, NULL},
 	{ USB_VID_DNP, USB_PID_DNP_DS80D, P_DNP_DS80D, NULL},
 	{ USB_VID_CITIZEN, USB_PID_CITIZEN_CW02, P_CITIZEN_OP900II, NULL},
