@@ -22,11 +22,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -1161,9 +1162,6 @@ static int kodak6800_main_loop(void *vctx, int copies) {
 			return CUPS_BACKEND_FAILED;
 		}
 
-		if (status.status == STATUS_IDLE)
-			break;
-
 		/* make sure we're not colliding with an existing
 		   jobid */
 		while (ctx->jobid == status.b1_jobid ||
@@ -1257,7 +1255,7 @@ static int kodak6800_main_loop(void *vctx, int copies) {
 /* Exported */
 struct dyesub_backend kodak6800_backend = {
 	.name = "Kodak 6800/6850",
-	.version = "0.58",
+	.version = "0.58.1",
 	.uri_prefix = "kodak6800",
 	.cmdline_usage = kodak6800_cmdline,
 	.cmdline_arg = kodak6800_cmdline_arg,
@@ -1270,7 +1268,7 @@ struct dyesub_backend kodak6800_backend = {
 	.devices = {
 	{ USB_VID_KODAK, USB_PID_KODAK_6800, P_KODAK_6800, "Kodak"},
 	{ USB_VID_KODAK, USB_PID_KODAK_6850, P_KODAK_6850, "Kodak"},
-	{ 0, 0, 0, ""}
+	{ 0, 0, 0, NULL}
 	}
 };
 
