@@ -18,11 +18,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -216,7 +217,7 @@ static int updr150_read_parse(void *vctx, int data_fd) {
 
 static int updr150_main_loop(void *vctx, int copies) {
 	struct updr150_ctx *ctx = vctx;
-	int i = 0, ret;
+	int i, ret;
 
 	if (!ctx)
 		return CUPS_BACKEND_FAILED;
@@ -228,6 +229,7 @@ static int updr150_main_loop(void *vctx, int copies) {
 	}
 
 top:
+	i = 0;
 	while (i < ctx->datalen) {
 		uint32_t len;
 		memcpy(&len, ctx->databuf + i, sizeof(len));
@@ -276,7 +278,7 @@ static int updr150_cmdline_arg(void *vctx, int argc, char **argv)
 
 struct dyesub_backend updr150_backend = {
 	.name = "Sony UP-DR150/UP-DR200/UP-CR10",
-	.version = "0.18",
+	.version = "0.19",
 	.uri_prefix = "sonyupdr150",
 	.cmdline_arg = updr150_cmdline_arg,
 	.init = updr150_init,
@@ -285,10 +287,10 @@ struct dyesub_backend updr150_backend = {
 	.read_parse = updr150_read_parse,
 	.main_loop = updr150_main_loop,
 	.devices = {
-	{ USB_VID_SONY, USB_PID_SONY_UPDR150, P_SONY_UPDR150, ""},
-	{ USB_VID_SONY, USB_PID_SONY_UPDR200, P_SONY_UPDR150, ""},
-	{ USB_VID_SONY, USB_PID_SONY_UPCR10, P_SONY_UPCR10, ""},
-	{ 0, 0, 0, ""}
+	{ USB_VID_SONY, USB_PID_SONY_UPDR150, P_SONY_UPDR150, NULL},
+	{ USB_VID_SONY, USB_PID_SONY_UPDR200, P_SONY_UPDR150, NULL},
+	{ USB_VID_SONY, USB_PID_SONY_UPCR10, P_SONY_UPCR10, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 
