@@ -14,8 +14,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Contents:
  *
@@ -461,6 +460,7 @@ main(int  argc,			    /* I - Number of command-line arguments */
   unsigned      rotor = 0;	    /* Rotor for generating PPD files in parallel */
   pid_t         *subprocesses = NULL;
   int		parent = 1;
+  int		skip_duplicate_ppds = 0;
 
  /*
   * Parse command-line args...
@@ -470,7 +470,7 @@ main(int  argc,			    /* I - Number of command-line arguments */
 
   for (;;)
   {
-    if ((i = getopt(argc, argv, "23hvqc:p:l:LMVd:saNCbZz")) == -1)
+    if ((i = getopt(argc, argv, "23hvqc:p:l:LMVd:saNCbZzS")) == -1)
       break;
 
     switch (i)
@@ -554,6 +554,9 @@ main(int  argc,			    /* I - Number of command-line arguments */
       use_compression = 0;
 #endif
       break;
+    case 'S':
+      skip_duplicate_ppds = 1;
+      break;
     default:
       usage();
       exit(EXIT_FAILURE);
@@ -659,9 +662,25 @@ main(int  argc,			    /* I - Number of command-line arguments */
     }
   else
     {
+      stp_string_list_t *seen_models = NULL;
+      if (skip_duplicate_ppds)
+	seen_models = stp_string_list_create();
+
       for (i = 0; i < stp_printer_model_count(); i++)
 	{
 	  printer = stp_get_printer_by_index(i);
+	  if (skip_duplicate_ppds)
+	    {
+	      char model_family[128];
+	      (void) snprintf(model_family, 127, "%d_%s",
+			      stp_printer_get_model(printer),
+			      stp_printer_get_family(printer));
+	      if (stp_string_list_is_present(seen_models, model_family))
+		continue;
+	      else
+		stp_string_list_add_string_unsafe(seen_models, model_family,
+						  model_family);
+	    }
 
 	  if (i % parallel == rotor && printer)
 	    {
@@ -672,6 +691,8 @@ main(int  argc,			    /* I - Number of command-line arguments */
 		return 1;
 	    }
 	}
+      if (seen_models)
+	stp_string_list_destroy(seen_models);
     }
   if (subprocesses)
     {
@@ -811,6 +832,7 @@ generate_ppd(
   return (status);
 }
 
+
 /*
  * 'help()' - Show detailed help.
  */
@@ -843,6 +865,7 @@ help(void)
        "  -z            Compress PPD files.\n"
        "  -Z            Don't compress PPD files.\n"
 #endif
+       "  -S            Skip PPD files with duplicate model identifiers.\n"
        "models:\n"
        "  A list of printer models, either the driver or quoted full name.\n");
 }
@@ -1106,7 +1129,8 @@ print_ppd_header(gpFile fp, ppd_type_t ppd_type, int model, const char *driver,
   gpputs(fp, "*% Copyright 1993-2008 by Mike Sweet and Robert Krawitz.\n");
   gpputs(fp, "*% This program is free software; you can redistribute it and/or\n");
   gpputs(fp, "*% modify it under the terms of the GNU General Public License,\n");
-  gpputs(fp, "*% version 2, as published by the Free Software Foundation.\n");
+  gpputs(fp, "*% either version 2, or (at your option) any later version,\n");
+  gpputs(fp, "*% as published by the Free Software Foundation.\n");
   gpputs(fp, "*%\n");
   gpputs(fp, "*% This program is distributed in the hope that it will be useful, but\n");
   gpputs(fp, "*% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n");
@@ -1114,8 +1138,7 @@ print_ppd_header(gpFile fp, ppd_type_t ppd_type, int model, const char *driver,
   gpputs(fp, "*% for more details.\n");
   gpputs(fp, "*%\n");
   gpputs(fp, "*% You should have received a copy of the GNU General Public License\n");
-  gpputs(fp, "*% along with this program; if not, write to the Free Software\n");
-  gpputs(fp, "*% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n");
+  gpputs(fp, "*% along with this program.  If not, see <https://www.gnu.org/licenses/>.\n");
   gpputs(fp, "*%\n");
   gpputs(fp, "*FormatVersion:	\"4.3\"\n");
   if (use_base_version)
@@ -1277,6 +1300,7 @@ print_page_sizes(gpFile fp, stp_vars_t *v, int simplified,
 	  continue;
 	}
       if (simplified && num_opts >= 10 &&
+	  (!desc.deflt.str || strcmp(opt->name, desc.deflt.str) != 0) &&
 	  (papersize->paper_unit == PAPERSIZE_ENGLISH_EXTENDED ||
 	   papersize->paper_unit == PAPERSIZE_METRIC_EXTENDED))
 	continue;
@@ -2045,8 +2069,10 @@ write_ppd(
     }
   stp_parameter_description_destroy(&desc);
 
-  if (stp_check_boolean_parameter(v, "NativeCopies", STP_PARAMETER_ACTIVE))
-    nativecopies = stp_get_boolean_parameter(v, "NativeCopies");
+  stp_describe_parameter(v, "NativeCopies", &desc);
+  if (desc.p_type == STP_PARAMETER_TYPE_BOOLEAN)
+    nativecopies = desc.deflt.boolean;
+  stp_parameter_description_destroy(&desc);
 
   if (nativecopies)
     gpputs(fp, "*cupsManualCopies: False\n");
@@ -2458,7 +2484,6 @@ write_ppd(
 	    print_group_close(fp, j, k, language, po);
 	}
     }
-  stp_parameter_list_destroy(param_list);
   stp_describe_parameter(v, "ImageType", &desc);
   if (desc.is_active && desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
     {
@@ -2527,6 +2552,7 @@ write_ppd(
 	      */
 
 	      if (simplified && num_opts >= 10 &&
+		  (!desc.deflt.str || strcmp(opt->name, desc.deflt.str) != 0) &&
 		  (papersize->paper_unit == PAPERSIZE_ENGLISH_EXTENDED ||
 		   papersize->paper_unit == PAPERSIZE_METRIC_EXTENDED))
 		continue;
@@ -2694,8 +2720,6 @@ write_ppd(
 	  gpprintf(fp, "*%s.StpiShrinkOutput %s/%s: \"\"\n", lang, "Crop", _("Crop (preserve dimensions)"));
 	  gpprintf(fp, "*%s.StpiShrinkOutput %s/%s: \"\"\n", lang, "Expand", _("Expand (use maximum page area)"));
 
-	  param_list = stp_get_parameter_list(v);
-
 	  for (j = 0; j <= STP_PARAMETER_CLASS_OUTPUT; j++)
 	    {
 	      for (k = 0; k <= maximum_level; k++)
@@ -2721,7 +2745,6 @@ write_ppd(
 		    }
 		}
 	    }
-	  stp_parameter_list_destroy(param_list);
 	  stp_describe_parameter(v, "ImageType", &desc);
 	  if (desc.is_active && desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
 	    {
@@ -2741,6 +2764,7 @@ write_ppd(
 	}
       po = savepo;
     }
+  stp_parameter_list_destroy(param_list);
   if (has_quality_parameter)
     stp_free(default_resolution);
   stp_string_list_destroy(resolutions);
