@@ -15,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Revision History:
  *
@@ -63,15 +62,6 @@ typedef struct
 
 typedef struct
 {
-  unsigned channel_count;
-  unsigned total_channels;
-  unsigned input_channels;
-  unsigned gcr_channels;
-  unsigned aux_output_channels;
-  size_t width;
-  int initialized;
-  unsigned ink_limit;
-  unsigned max_density;
   stpi_channel_t *c;
   stp_curve_t *gcr_curve;
   unsigned curve_count;
@@ -84,12 +74,23 @@ typedef struct
   unsigned short *alloc_data_1;
   unsigned short *alloc_data_2;
   unsigned short *alloc_data_3;
-  int black_channel;
-  int gloss_channel;
-  int gloss_physical_channel;
+  unsigned char *output_data_8bit;
+  size_t width;
   double cyan_balance;
   double magenta_balance;
   double yellow_balance;
+  unsigned channel_count;
+  unsigned total_channels;
+  unsigned input_channels;
+  unsigned gcr_channels;
+  unsigned aux_output_channels;
+  unsigned ink_limit;
+  unsigned max_density;
+  int black_channel;
+  int gloss_channel;
+  int gloss_physical_channel;
+  int initialized;
+  int valid_8bit;
 } stpi_channel_group_t;
 
 
@@ -141,6 +142,7 @@ stpi_channel_clear(void *vc)
   cg->total_channels = 0;
   cg->input_channels = 0;
   cg->initialized = 0;
+  cg->valid_8bit = 0;
 }
 
 void
@@ -465,6 +467,51 @@ input_needs_splitting(const stp_vars_t *v)
 #endif
 }
 
+static void
+stp_dump_channels(const stp_vars_t *v)
+{
+  stpi_channel_group_t *cg = get_channel_group(v);
+  int i, j;
+  stp_erprintf("   channel_count  %d\n", cg->channel_count);
+  stp_erprintf("   total_channels %d\n", cg->total_channels);
+  stp_erprintf("   input_channels %d\n", cg->input_channels);
+  stp_erprintf("   aux_channels   %d\n", cg->aux_output_channels);
+  stp_erprintf("   gcr_channels   %d\n", cg->gcr_channels);
+  stp_erprintf("   width          %ld\n", (long)cg->width);
+  stp_erprintf("   ink_limit      %d\n", cg->ink_limit);
+  stp_erprintf("   gloss_limit    %d\n", cg->gloss_limit);
+  stp_erprintf("   max_density    %d\n", cg->max_density);
+  stp_erprintf("   curve_count    %d\n", cg->curve_count);
+  stp_erprintf("   black_channel  %d\n", cg->black_channel);
+  stp_erprintf("   gloss_channel  %d\n", cg->gloss_channel);
+  stp_erprintf("   gloss_physical %d\n", cg->gloss_physical_channel);
+  stp_erprintf("   cyan           %.3f\n", cg->cyan_balance);
+  stp_erprintf("   magenta        %.3f\n", cg->magenta_balance);
+  stp_erprintf("   yellow         %.3f\n", cg->yellow_balance);
+  stp_erprintf("   input_data     %p\n", (void *) cg->input_data);
+  stp_erprintf("   multi_tmp      %p\n", (void *) cg->multi_tmp);
+  stp_erprintf("   split_input    %p\n", (void *) cg->split_input);
+  stp_erprintf("   output_data    %p\n", (void *) cg->output_data);
+  stp_erprintf("   gcr_data       %p\n", (void *) cg->gcr_data);
+  stp_erprintf("   alloc_data_1   %p\n", (void *) cg->alloc_data_1);
+  stp_erprintf("   alloc_data_2   %p\n", (void *) cg->alloc_data_2);
+  stp_erprintf("   alloc_data_3   %p\n", (void *) cg->alloc_data_3);
+  stp_erprintf("   gcr_curve      %p\n", (void *) cg->gcr_curve);
+  for (i = 0; i < cg->channel_count; i++)
+    {
+      stp_erprintf("   Channel %d:\n", i);
+      for (j = 0; j < cg->c[i].subchannel_count; j++)
+	{
+	  stpi_subchannel_t *sch = &(cg->c[i].sc[j]);
+	  stp_erprintf("      Subchannel %d:\n", j);
+	  stp_erprintf("         value   %.3f:\n", sch->value);
+	  stp_erprintf("         lower   %.3f:\n", sch->lower);
+	  stp_erprintf("         upper   %.3f:\n", sch->upper);
+	  stp_erprintf("         cutoff  %.3f:\n", sch->cutoff);
+	  stp_erprintf("         density %d:\n", sch->s_density);
+	}
+    }
+}
 
 void
 stp_channel_initialize(stp_vars_t *v, stp_image_t *image,
@@ -616,54 +663,8 @@ stp_channel_initialize(stp_vars_t *v, stp_image_t *image,
   cg->magenta_balance = stp_get_float_parameter(v, "MagentaBalance");
   cg->yellow_balance = stp_get_float_parameter(v, "YellowBalance");
   stp_dprintf(STP_DBG_INK, v, "stp_channel_initialize:\n");
-  stp_dprintf(STP_DBG_INK, v, "   channel_count  %d\n", cg->channel_count);
-  stp_dprintf(STP_DBG_INK, v, "   total_channels %d\n", cg->total_channels);
-  stp_dprintf(STP_DBG_INK, v, "   input_channels %d\n", cg->input_channels);
-  stp_dprintf(STP_DBG_INK, v, "   aux_channels   %d\n", cg->aux_output_channels);
-  stp_dprintf(STP_DBG_INK, v, "   gcr_channels   %d\n", cg->gcr_channels);
-  stp_dprintf(STP_DBG_INK, v, "   width          %ld\n", (long)cg->width);
-  stp_dprintf(STP_DBG_INK, v, "   ink_limit      %d\n", cg->ink_limit);
-  stp_dprintf(STP_DBG_INK, v, "   gloss_limit    %d\n", cg->gloss_limit);
-  stp_dprintf(STP_DBG_INK, v, "   max_density    %d\n", cg->max_density);
-  stp_dprintf(STP_DBG_INK, v, "   curve_count    %d\n", cg->curve_count);
-  stp_dprintf(STP_DBG_INK, v, "   black_channel  %d\n", cg->black_channel);
-  stp_dprintf(STP_DBG_INK, v, "   gloss_channel  %d\n", cg->gloss_channel);
-  stp_dprintf(STP_DBG_INK, v, "   gloss_physical %d\n", cg->gloss_physical_channel);
-  stp_dprintf(STP_DBG_INK, v, "   cyan           %.3f\n", cg->cyan_balance);
-  stp_dprintf(STP_DBG_INK, v, "   magenta        %.3f\n", cg->magenta_balance);
-  stp_dprintf(STP_DBG_INK, v, "   yellow         %.3f\n", cg->yellow_balance);
-  stp_dprintf(STP_DBG_INK, v, "   input_data     %p\n",
-	      (void *) cg->input_data);
-  stp_dprintf(STP_DBG_INK, v, "   multi_tmp      %p\n",
-	      (void *) cg->multi_tmp);
-  stp_dprintf(STP_DBG_INK, v, "   split_input    %p\n",
-	      (void *) cg->split_input);
-  stp_dprintf(STP_DBG_INK, v, "   output_data    %p\n",
-	      (void *) cg->output_data);
-  stp_dprintf(STP_DBG_INK, v, "   gcr_data       %p\n",
-	      (void *) cg->gcr_data);
-  stp_dprintf(STP_DBG_INK, v, "   alloc_data_1   %p\n",
-	      (void *) cg->alloc_data_1);
-  stp_dprintf(STP_DBG_INK, v, "   alloc_data_2   %p\n",
-	      (void *) cg->alloc_data_2);
-  stp_dprintf(STP_DBG_INK, v, "   alloc_data_3   %p\n",
-	      (void *) cg->alloc_data_3);
-  stp_dprintf(STP_DBG_INK, v, "   gcr_curve      %p\n",
-	      (void *) cg->gcr_curve);
-  for (i = 0; i < cg->channel_count; i++)
-    {
-      stp_dprintf(STP_DBG_INK, v, "   Channel %d:\n", i);
-      for (j = 0; j < cg->c[i].subchannel_count; j++)
-	{
-	  stpi_subchannel_t *sch = &(cg->c[i].sc[j]);
-	  stp_dprintf(STP_DBG_INK, v, "      Subchannel %d:\n", j);
-	  stp_dprintf(STP_DBG_INK, v, "         value   %.3f:\n", sch->value);
-	  stp_dprintf(STP_DBG_INK, v, "         lower   %.3f:\n", sch->lower);
-	  stp_dprintf(STP_DBG_INK, v, "         upper   %.3f:\n", sch->upper);
-	  stp_dprintf(STP_DBG_INK, v, "         cutoff  %.3f:\n", sch->cutoff);
-	  stp_dprintf(STP_DBG_INK, v, "         density %d:\n", sch->s_density);
-	}
-    }
+  if (stp_get_debug_level() & STP_DBG_INK)
+    stp_dump_channels(v);
 }
 
 static void
@@ -738,6 +739,7 @@ limit_ink(const stp_vars_t *v)
   unsigned short *ptr;
   if (!cg || cg->ink_limit == 0 || cg->ink_limit >= cg->max_density)
     return 0;
+  cg->valid_8bit = 0;
   ptr = cg->output_data;
   for (i = 0; i < cg->width; i++)
     {
@@ -854,6 +856,7 @@ generate_special_channels(const stp_vars_t *v)
   int outbytes;
   if (!cg)
     return;
+  cg->valid_8bit = 0;
   input = cg->input_data;
   output = cg->multi_tmp;
   offset = (cg->black_channel >= 0 ? 0 : -1);
@@ -929,6 +932,7 @@ split_channels(const stp_vars_t *v, unsigned *zero_mask)
   unsigned short *output;
   if (!cg)
     return;
+  cg->valid_8bit = 0;
   outbytes = cg->total_channels * sizeof(unsigned short);
   input = cg->split_input;
   output = cg->output_data;
@@ -1024,6 +1028,7 @@ scale_channels(const stp_vars_t *v, unsigned *zero_mask)
   int physical_channel = 0;
   if (!cg)
     return;
+  cg->valid_8bit = 0;
   if (zero_mask)
     *zero_mask = 0;
   for (i = 0; i < cg->channel_count; i++)
@@ -1070,6 +1075,7 @@ generate_gloss(const stp_vars_t *v, unsigned *zero_mask)
   int i, j, k;
   if (!cg || cg->gloss_channel == -1 || cg->gloss_limit <= 0)
     return;
+  cg->valid_8bit = 0;
   output = cg->output_data;
   gloss_mask = ~(1 << cg->gloss_physical_channel);
   for (i = 0; i < cg->width; i++)
@@ -1116,6 +1122,7 @@ do_gcr(const stp_vars_t *v)
 
   if (!cg)
     return;
+  cg->valid_8bit = 0;
 
   output = cg->gcr_data;
   stp_curve_resample(cg->gcr_curve, 65536);
@@ -1173,3 +1180,23 @@ stp_channel_get_output(const stp_vars_t *v)
     return NULL;
   return cg->output_data;
 }
+
+unsigned char *
+stp_channel_get_output_8bit(const stp_vars_t *v)
+{
+  stpi_channel_group_t *cg = get_channel_group(v);
+  if (!cg)
+    return NULL;
+  if (cg->valid_8bit)
+    return cg->output_data_8bit;
+  if (! cg->output_data_8bit)
+    cg->output_data_8bit = stp_malloc(sizeof(unsigned char) * 
+				      cg->total_channels * cg->width);
+  int i;
+  (void) memset(cg->output_data_8bit, 0, sizeof(unsigned char) * 
+		cg->total_channels * cg->width);
+  for (i = 0; i < cg->width * cg->total_channels; i++) 
+    cg->output_data_8bit[i] = cg->output_data[i] / (unsigned short) 257;
+  cg->valid_8bit = 1;
+  return cg->output_data_8bit;
+}
